if(typeof Promise == 'undefined') {
    global.Promise = require('promise-polyfill')
}


var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var markdown = require('express-markdown');

var routes = require('./routes/index');
var iskanje = require('./routes/iskanje');
var organ = require('./routes/organ');
var podj = require("./routes/podj");
var rpusearch = require('./routes/rpusearch');
var prssearch = require('./routes/prssearch');
var razmerje = require('./routes/razmerje');
var mesec = require('./routes/mesec');
var ajax = require('./routes/ajax');
var api = require('./routes/api');
var priljubljene = require('./routes/priljubljene');
var statistike = require('./routes/statistike');
var streznik = require('./routes/streznik');
var user = require('./routes/user');

var app = express();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// uncomment after placing your favicon in /public
//app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(markdown({ directory: __dirname + '/public/markdown', view: 'markdown',variable: 'markdown'}));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());

var options = {
  dotfiles: 'ignore',
  etag: false,
  extensions: ['htm', 'html'],
  index: false,
  maxAge: '1d',
  redirect: false,
  setHeaders: function (res, path, stat) {
    res.set('x-timestamp', Date.now());
  }
}

app.use(express.static(path.join(__dirname, 'public'), options));

app.use('/', routes);
app.use('/iskanje', iskanje);
app.use('/organ', organ);
app.use('/podj', podj);
app.use('/rpusearch', rpusearch);
app.use('/prssearch', prssearch);
app.use('/razmerje', razmerje);
app.use('/mesec', mesec);
app.use('/ajax', ajax);
app.use('/api', api);
app.use('/priljubljene', priljubljene);
app.use('/statistike', statistike);
app.use('/streznik', streznik);
app.use('/user', user);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
  var err = new Error('Not Found');
  err.status = 404;
  next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
  app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    res.render('error', {
      message: err.message,
      error: err
    });
  });
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
  res.status(err.status || 500);
  res.render('error', {
    message: err.message,
    error: {}
  });
});


module.exports = app;
